==> pressio4py-0.7.0rc1.tar.gz: pressio4py-0.7.0rc1/pressio/packages/containers/src/expressions/subspan/containers_subspan_function.hpp <==
/*
//@HEADER
// ************************************************************************
//
// containers_subspan_function.hpp
//                     		  Pressio
//                             Copyright 2019
//    National Technology & Engineering Solutions of Sandia, LLC (NTESS)
//
// Under the terms of Contract DE-NA0003525 with NTESS, the
// U.S. Government retains certain rights in this software.
//
// Pressio is licensed under BSD-3-Clause terms of use:
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its
// contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Francesco Rizzi (fnrizzi@sandia.gov)
//
// ************************************************************************
//@HEADER
*/

#ifndef CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_
#define CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_

namespace pressio{ namespace containers{

template <typename T, typename ... Args>
mpl::enable_if_t<
  ::pressio::containers::predicates::is_dense_matrix_wrapper<T>::value
  and (0 < sizeof...(Args)),
  typename details::traits<T>::subspan_const_ret_t
  >
subspan(const T & obj, Args&& ... args)
{
  using return_t = typename details::traits<T>::subspan_const_ret_t;
  return return_t(obj, std::forward<Args>(args)... );
}

template <typename T, typename ... Args>
mpl::enable_if_t<
  ::pressio::containers::predicates::is_dense_matrix_wrapper<T>::value
  and (0 < sizeof...(Args)),
  typename details::traits<T>::subspan_ret_t
  >
subspan(T & obj, Args&& ... args)
{
  using return_t = typename details::traits<T>::subspan_ret_t;
  return return_t(obj, std::forward<Args>(args)... );
}

}} //end namespace pressio::containers
#endif  // CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_

==> pressio4py-0.8.0.tar.gz: pressio4py-0.8.0/pressio/packages/containers/src/expressions/subspan/containers_subspan_function.hpp <==
/*
//@HEADER
// ************************************************************************
//
// containers_subspan_function.hpp
//                     		  Pressio
//                             Copyright 2019
//    National Technology & Engineering Solutions of Sandia, LLC (NTESS)
//
// Under the terms of Contract DE-NA0003525 with NTESS, the
// U.S. Government retains certain rights in this software.
//
// Pressio is licensed under BSD-3-Clause terms of use:
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its
// contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Francesco Rizzi (fnrizzi@sandia.gov)
//
// ************************************************************************
//@HEADER
*/

#ifndef CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_
#define CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_

namespace pressio{ namespace containers{

template <typename T, typename ... Args>
mpl::enable_if_t<
  (::pressio::containers::predicates::is_dense_matrix_wrapper<T>::value
   or (::pressio::containers::predicates::is_tensor_wrapper<T>::value and T::traits::rank==2))
  and (0 < sizeof...(Args)),
  typename details::traits<T>::subspan_const_ret_t
  >
subspan(const T & obj, Args&& ... args)
{
  using return_t = typename details::traits<T>::subspan_const_ret_t;
  return return_t(obj, std::forward<Args>(args)... );
}

template <typename T, typename ... Args>
mpl::enable_if_t<
  (::pressio::containers::predicates::is_dense_matrix_wrapper<T>::value
   or (::pressio::containers::predicates::is_tensor_wrapper<T>::value and T::traits::rank==2))
  and (0 < sizeof...(Args)),
  typename details::traits<T>::subspan_ret_t
  >
subspan(T & obj, Args&& ... args)
{
  using return_t = typename details::traits<T>::subspan_ret_t;
  return return_t(obj, std::forward<Args>(args)... );
}

}} //end namespace pressio::containers
#endif  // CONTAINERS_EXPRESSIONS_SUBSPAN_CONTAINERS_SUBSPAN_FUNCTION_HPP_
